@@ -1,6 +1,10 @@
+import uuid
+
+import requests
+
 from client import exceptions as ex
 from client.sources.common import core
-from client.utils import auth, format
+from client.utils import auth, format, encryption
 from client.protocols.grading import grade
 import client
 import collections
@@ -11,6 +15,8 @@ import logging
 import os
 import textwrap
 
+from client.utils.printer import print_success, print_error, print_warning
+
 log = logging.getLogger(__name__)
 
 CONFIG_EXTENSION = '*.ok'
@@ -57,6 +63,7 @@ def _get_config(config):
 class Assignment(core.Serializable):
     name = core.String()
     endpoint = core.String(optional=True, default='')
+    decryption_keypage = core.String(optional=True, default='')
     src = core.List(type=str, optional=True)
     tests = core.Dict(keys=str, values=str, ordered=True)
     default_tests = core.List(type=str, optional=True)
@@ -102,6 +109,122 @@ class Assignment(core.Serializable):
         grade(tests, messages, env)
         return messages['grading'][test_name]
 
+    ##############
+    # Encryption #
+    ##############
+
+    def generate_encryption_key(self, keys_file):
+        data = [(filename, encryption.generate_key()) for filename in self._get_files()]
+        with open(keys_file, "w") as f:
+            json.dump(data, f)
+
+    def encrypt(self, keys_file, padding):
+        """
+        Encrypt each question and test, with the given keys file, which contains (file, key) pairs
+        """
+        with open(keys_file) as f:
+            keys = dict(json.load(f))
+        for file in self._get_files():
+            if file in keys:
+                self._encrypt_file(file, keys[file], padding)
+
+    def decrypt(self, keys):
+        decrypted_files, undecrypted_files = self.attempt_decryption(keys)
+        if not undecrypted_files + decrypted_files:
+            print_success("All files are decrypted")
+        elif undecrypted_files:
+            if keys:
+                print_error("Unable to decrypt some files with the keys", ", ".join(keys))
+            else:
+                print_error("No keys found, could not decrypt any files")
+            print_error("    Non-decrypted files:", *undecrypted_files)
+
+    def attempt_decryption(self, keys):
+        if self.decryption_keypage:
+            try:
+                response = requests.get(self.decryption_keypage)
+                response.raise_for_status()
+                keys_data = response.content.decode('utf-8')
+                keys = keys + encryption.get_keys(keys_data)
+            except Exception as e:
+                print_error(
+                    "Could not load decryption page {}: {}.".format(self.decryption_keypage, e))
+                print_error("You can pass in a key directly by running python3 ok --decrypt [KEY]")
+
+        decrypted_files = []
+        undecrypted_files = []
+        for file in self._get_files():
+            with open(file) as f:
+                if not encryption.is_encrypted(f.read()):
+                    continue
+            for key in keys:
+                success = self._decrypt_file(file, key)
+                if success:
+                    decrypted_files.append(file)
+                    break
+            else:
+                undecrypted_files.append(file)
+        return decrypted_files, undecrypted_files
+
+    def _decrypt_file(self, path, key):
+        """
+        Decrypt the given file in place with the given key.
+        If the key does not match, do not change the file contents
+        """
+        success = False
+
+        def decrypt(ciphertext):
+            if not encryption.is_encrypted(ciphertext):
+                return ciphertext
+            try:
+                plaintext = encryption.decrypt(ciphertext, key)
+                nonlocal success
+                success = True
+                print_success("decrypted", path, "with", key)
+                return plaintext
+            except encryption.InvalidKeyException:
+                return ciphertext
+
+        self._in_place_edit(path, decrypt)
+        return success
+
+    def _encrypt_file(self, path, key, padding):
+        """
+        Encrypt the given file in place with the given key.
+        This is idempotent but if you try to encrypt the same file with multiple keys it errors.
+        """
+        def encrypt(data):
+            if encryption.is_encrypted(data):
+                try:
+                    data = encryption.decrypt(data, key)
+                except encryption.InvalidKeyException:
+                    raise ValueError("Attempt to re-encrypt file with an invalid key")
+            return encryption.encrypt(data, key, padding)
+
+        self._in_place_edit(path, encrypt)
+
+    @staticmethod
+    def _in_place_edit(path, func):
+        """
+        Edit the given file in place, atomically. `func` is a function that modifies the data in the file.
+        """
+        with open(path) as f:
+            data = f.read()
+        ciphertext = func(data)
+        temporary_file = "." + uuid.uuid4().hex
+        with open(temporary_file, "w") as f:
+            f.write(ciphertext)
+        # atomic rename
+        os.replace(temporary_file, path)
+
+    def _get_files(self):
+        """
+        Get all the test and submission source files associated with this assignment, deduplicated
+        """
+        tests = [file for k, v in self.tests.items() for file in glob.glob(k) if v == 'ok_test' or v == 'scheme_test']
+        src = list(self.src)
+        return sorted(set(tests + src))
+
     @property
     def server_url(self):
         scheme = 'http' if self.cmd_args.insecure else 'https'
@@ -126,11 +249,12 @@ class Assignment(core.Serializable):
     # lock          -> none
     # rate_limit    -> none
     # scoring       -> none
+    # trace         -> file_contents
     # unlock        -> none
     # testing       -> none
     _PROTOCOLS = [
         "testing",
-        "rate_limit",
+        # "rate_limit", uncomment to turn rate limiting back on!
         "file_contents",
         "grading",
         "analytics",
@@ -140,6 +264,7 @@ class Assignment(core.Serializable):
         "lock",
         "scoring",
         "unlock",
+        "trace",
         "backup",
     ]
 
@@ -179,32 +304,41 @@ class Assignment(core.Serializable):
     def get_identifier(self):
         return auth.get_identifier(self.cmd_args, endpoint=self.endpoint)
 
+    def is_empty_init(self, path):
+        if os.path.basename(path) != '__init__.py':
+            return False
+
+        with open(path) as f:
+            contents = f.read()
+
+        return contents.strip() == ""
+
     def _load_tests(self):
         """Loads all tests specified by test_map."""
         log.info('Loading tests')
-        for file_pattern, source in self.tests.items():
-            # Separate filepath and parameter
-            if ':' in file_pattern:
-                file_pattern, parameter = file_pattern.split(':', 1)
-            else:
-                parameter = ''
+        for file_pattern, sources in self.tests.items():
+            for source in sources.split(","):
+                # Separate filepath and parameter
+                if ':' in file_pattern:
+                    file_pattern, parameter = file_pattern.split(':', 1)
+                else:
+                    parameter = ''
 
-            for file in sorted(glob.glob(file_pattern)):
-                try:
-                    module = importlib.import_module(self._TESTS_PACKAGE + '.' + source)
-                except ImportError:
-                    raise ex.LoadingException('Invalid test source: {}'.format(source))
+                for file in sorted(glob.glob(file_pattern)):
+                    if self.is_empty_init(file):
+                        continue
+                    try:
+                        module = importlib.import_module(self._TESTS_PACKAGE + '.' + source)
+                    except ImportError:
+                        raise ex.LoadingException('Invalid test source: {}'.format(source))
 
-                test_name = file
-                if parameter:
-                    test_name += ':' + parameter
+                    test_name = file
+                    if parameter:
+                        test_name += ':' + parameter
 
-                self.test_map.update(module.load(file, parameter, self))
+                    self.test_map.update(module.load(file, parameter, self))
 
-                log.info('Loaded {}'.format(test_name))
-
-        if not self.test_map:
-            raise ex.LoadingException('No tests loaded')
+                    log.info('Loaded {}'.format(test_name))
 
     def dump_tests(self):
         """Dumps all tests, as determined by their .dump() method.
@@ -235,6 +369,11 @@ class Assignment(core.Serializable):
                 and len(self.default_tests) > 0:
             log.info('Using default tests (no questions specified): '
                      '{}'.format(self.default_tests))
+            bad_tests = sorted(test for test in self.default_tests if test not in self.test_map)
+            if bad_tests:
+                error_message = ("Required question(s) missing: {}. "
+                    "This often is the result of accidentally deleting the question's doctests or the entire function.")
+                raise ex.LoadingException(error_message.format(", ".join(bad_tests)))
             return [self.test_map[test] for test in self.default_tests]
         elif not questions:
             log.info('Using all tests (no questions specified and no default tests)')
@@ -246,12 +385,7 @@ class Assignment(core.Serializable):
         specified_tests = []
         for question in questions:
             if question not in self.test_map:
-                print('Test "{}" not found.'.format(question))
-                print('Did you mean one of the following? '
-                      '(Names are case sensitive)')
-                for test in self.test_map:
-                    print('    {}'.format(test))
-                raise ex.LoadingException('Invalid test specified: {}'.format(question))
+                raise ex.InvalidTestInQuestionListException(list(self.test_map), question)
 
             log.info('Adding {} to specified tests'.format(question))
             if question not in specified_tests:
