@@ -57,7 +57,11 @@ import logging
 import os
 import sys
 import struct
+import colorama
 
+from client.utils.printer import print_error
+
+colorama.init()
 LOGGING_FORMAT = '%(levelname)s  | %(filename)s:%(lineno)d | %(message)s'
 logging.basicConfig(format=LOGGING_FORMAT)
 log = logging.getLogger('client')   # Get top-level logger
@@ -89,7 +93,7 @@ def parse_input(command_input=None):
                         help="start the Python interpreter after a failed test")
     testing.add_argument('-v', '--verbose', action='store_true',
                         help="show all tests, not just passing tests")
-    testing.add_argument('-t', '--testing', nargs='?', type=str, const='mytests.rst', 
+    testing.add_argument('-t', '--testing', nargs='?', type=str, const='mytests.rst',
                         help='run tests from rst file (default: mytests.rst)')
     testing.add_argument('--all', action='store_true',
                         help="run tests for all questions in config file")
@@ -104,6 +108,14 @@ def parse_input(command_input=None):
     testing.add_argument('-cov', '--coverage', action='store_true',
                         help="get suggestions on what lines to add tests for")
 
+    # Debugging
+    debugging = parser.add_argument_group('debugging tools for students')
+
+    debugging.add_argument('--trace', action='store_true',
+                         help="trace code and launch python tutor")
+    debugging.add_argument('--trace-print', action='store_true',
+                         help="print the trace instead of visualizing it")
+
     # Experiments
     experiment = parser.add_argument_group('experiment options')
     experiment.add_argument('--no-experiments', action='store_true',
@@ -116,7 +128,7 @@ def parse_input(command_input=None):
                         help="launch collaborative programming environment")
 
     # Debug information
-    debug = parser.add_argument_group('debugging options')
+    debug = parser.add_argument_group('ok developer debugging options')
     debug.add_argument('--version', action='store_true',
                         help="print the version number and exit")
     debug.add_argument('--tests', action='store_true',
@@ -136,6 +148,18 @@ def parse_input(command_input=None):
     grading.add_argument('--config', type=str,
                         help="use a specific configuration file")
 
+    # Encrypt
+    crypt = parser.add_argument_group('encryption')
+    crypt.add_argument('--generate-encryption-key', type=str,
+                       help='generates a JSON file containing a list of [(file, key)] pairs. Path is a keyfile')
+    crypt.add_argument('--encrypt', type=str,
+                       help='encrypt each problem. provide a path to a keyfile generated by --generate-encryption-key')
+    crypt.add_argument('--encrypt-padding', type=int,
+                       help='If provided, pads all plaintexts to this size in bytes.'
+                            'Errors if any of the files are longer than this')
+    crypt.add_argument('--decrypt', type=str, nargs='*',
+                       help='decrypt all problems where the given keys apply')
+
     # Server parameters
     server = parser.add_argument_group('server options')
     server.add_argument('--local', action='store_true',
@@ -187,10 +211,18 @@ def main():
         # Instantiating assignment
         assign = assignment.load_assignment(args.config, args)
 
-        if args.authenticate:
-            # Authenticate and check for success
-            if not assign.authenticate(force=True):
-                exit(1)
+        if args.generate_encryption_key:
+            assign.generate_encryption_key(args.generate_encryption_key)
+            exit(0)
+
+        if args.encrypt:
+            assign.encrypt(args.encrypt, args.encrypt_padding)
+            # do not dump tests back out, this overwrites any changes that may have been made
+            assign = None
+            exit(0)
+
+        if args.decrypt is not None:
+            raise ex.ForceDecryptionException(args.decrypt)
 
         if args.tests:
             print('Available tests:')
@@ -198,25 +230,52 @@ def main():
                 print('    ' + name)
             exit(0)
 
-        msgs = messages.Messages()
-        for name, proto in assign.protocol_map.items():
-            log.info('Execute {}.run()'.format(name))
-            proto.run(msgs)
+        force_authenticate = args.authenticate
+        retry = True
+        while retry:
+            retry = False
+            if force_authenticate:
+                # Authenticate and check for success
+                if not assign.authenticate(force=True):
+                    exit(1)
 
-        msgs['timestamp'] = str(datetime.now())
+            try:
+                msgs = messages.Messages()
+                for name, proto in assign.protocol_map.items():
+                    log.info('Execute {}.run()'.format(name))
+                    proto.run(msgs)
+                msgs['timestamp'] = str(datetime.now())
+            except ex.AuthenticationException as e:
+                if not force_authenticate:
+                    force_authenticate = True
+                    retry = True
+                elif not args.no_browser:
+                    args.no_browser = True
+                    retry = True
+                if retry:
+                    msg = "without a browser" if args.no_browser else "with a browser"
+                    log.warning('Authentication exception occurred; will retry {0}'.format(msg), exc_info=True)
+                    print_error('Authentication error; will try to re-authenticate {0}...'.format(msg))
+                else:
+                    raise  # outer handler will be called
 
+    except ex.ForceDecryptionException as e:
+        assign.decrypt(e.keys)
+        # do not dump tests back out, this could overwrite any changes that may have been made
+        assign = None
+        exit(0)
     except ex.LoadingException as e:
         log.warning('Assignment could not load', exc_info=True)
-        print('Error loading assignment: ' + str(e))
+        print_error('Error loading assignment: ' + str(e))
     except ex.AuthenticationException as e:
         log.warning('Authentication exception occurred', exc_info=True)
-        print('Authentication error: {0}'.format(e))
+        print_error('Authentication error: {0}'.format(e))
     except ex.EarlyExit as e:
         log.warning('OK exited early (non-error)')
-        print(str(e))
+        print_error(str(e))
     except ex.OkException as e:
         log.warning('General OK exception occurred', exc_info=True)
-        print('Error: ' + str(e))
+        print_error('Error: ' + str(e))
     except KeyboardInterrupt:
         log.info('KeyboardInterrupt received.')
     finally:
