@@ -9,6 +9,8 @@ import client
 import logging
 import os.path
 
+from client.utils.printer import print_error
+
 LOGGING_FORMAT = '%(levelname)s | pid %(process)d | %(filename)s:%(lineno)d | %(message)s'
 logging.basicConfig(format=LOGGING_FORMAT)
 log = logging.getLogger('client')   # Get top-level logger
@@ -31,7 +33,7 @@ def main():
         scoring.protocol(args, assign).run(msgs)
     except (ex.LoadingException, ex.SerializeException) as e:
         log.warning('Assignment could not instantiate', exc_info=True)
-        print('Error: ' + str(e).strip())
+        print_error('Error: ' + str(e).strip())
         exit(1)
     except (KeyboardInterrupt, EOFError):
         log.info('Quitting...')
