@@ -1,3 +1,6 @@
+from display_timedelta import display_timedelta
+
+from client import exceptions
 from client.protocols.common import models
 from client.utils import network
 import client
@@ -9,6 +12,8 @@ import requests
 
 log = logging.getLogger(__name__)
 
+from client.utils.printer import print_warning, print_success, print_error
+
 class BackupProtocol(models.Protocol):
 
     # Timeouts are specified in seconds.
@@ -18,6 +23,7 @@ class BackupProtocol(models.Protocol):
     BACKUP_FILE = ".ok_messages"
     BACKUP_ENDPOINT = '{server}/api/v3/backups/'
     REVISION_ENDPOINT = '{server}/api/v3/revision/'
+    ASSIGNMENT_ENDPOINT = '{server}/api/v3/assignment/'
 
     def run(self, messages):
         if not self.assignment.endpoint:
@@ -25,7 +31,7 @@ class BackupProtocol(models.Protocol):
             return
 
         if self.args.local:
-            print("Cannot backup when running ok with --local.")
+            print_warning("Cannot backup when running ok with --local.")
             return
 
         if not self.args.insecure:
@@ -45,7 +51,7 @@ class BackupProtocol(models.Protocol):
         log.info('Sending unsent messages')
 
         if not access_token:
-            print("Not authenticated. Cannot send {} to server".format(action))
+            print_error("Not authenticated. Cannot send {} to server".format(action))
             self.dump_unsent_messages(message_list)
             return
 
@@ -67,7 +73,7 @@ class BackupProtocol(models.Protocol):
         base_url = self.assignment.server_url + '/{}/{}/{}'
 
         if isinstance(response, dict):
-            print('{action} successful for user: {email}'.format(action=action.title(),
+            print_success('{action} successful for user: {email}'.format(action=action.title(),
                         email=response['data']['email']))
 
             submission_type = 'submissions' if self.args.submit else 'backups'
@@ -76,7 +82,7 @@ class BackupProtocol(models.Protocol):
                                   response['data']['key'])
 
             if self.args.submit or self.args.backup:
-                print('URL: {0}'.format(url))
+                print_success('URL: {0}'.format(url))
 
             if self.args.backup:
                 print('NOTE: this is only a backup. '
@@ -177,7 +183,9 @@ class BackupProtocol(models.Protocol):
                 log.warning('%s error message: %s', ex.__class__.__name__,
                             response_json['message'])
 
-                if ex.response.status_code == 403 and 'download_link' in response_json['data']:
+                if ex.response.status_code == 401:  # UNAUTHORIZED (technically authorization != authentication, but oh well)
+                    raise exceptions.AuthenticationException(response_json.get('message'))  # raise this for the caller
+                elif ex.response.status_code == 403 and 'download_link' in response_json['data']:
                     retries = 0
                     error_msg = 'Aborting because OK may need to be updated.'
                 else:
@@ -200,9 +208,17 @@ class BackupProtocol(models.Protocol):
 
         if current and error_msg:
             print()     # Preserve progress bar.
-            print('Could not', action.lower() + ':', error_msg)
+            print_error('Could not', action.lower() + ':', error_msg)
         elif not message_list:
             print('{action}... 100% complete'.format(action=action))
+            due_date = self.get_due_date(access_token, timeout)
+            if due_date is not None:
+                now = datetime.datetime.now(tz=datetime.timezone.utc)
+                time_to_deadline = due_date - now
+                if time_to_deadline < datetime.timedelta(0):
+                    print_error("{action} past deadline by".format(action=action), display_timedelta(-time_to_deadline))
+                elif time_to_deadline < datetime.timedelta(hours=10):
+                    print_warning("Assignment is due in", display_timedelta(time_to_deadline))
             return first_response
         elif not send_all:
             # Do not display any error messages if --backup or --submit are not
@@ -211,11 +227,11 @@ class BackupProtocol(models.Protocol):
         elif not error_msg:
             # No errors occurred, but could not complete request within TIMEOUT.
             print()     # Preserve progress bar.
-            print('Could not {} within {} seconds.'.format(action.lower(), timeout))
+            print_error('Could not {} within {} seconds.'.format(action.lower(), timeout))
         else:
             # If not all messages could be backed up successfully.
             print()     # Preserve progress bar.
-            print('Could not', action.lower() + ':', error_msg)
+            print_error('Could not', action.lower() + ':', error_msg)
 
     def send_messages(self, access_token, messages, timeout, current):
         """Send messages to server, along with user authentication."""
@@ -246,4 +262,16 @@ class BackupProtocol(models.Protocol):
         response.raise_for_status()
         return response.json()
 
+    def get_due_date(self, access_token, timeout):
+        address = self.ASSIGNMENT_ENDPOINT.format(server=self.assignment.server_url) + self.assignment.endpoint
+        response = requests.get(address,
+                                headers={'Authorization': 'Bearer {}'.format(access_token)},
+                                timeout=timeout)
+        response_data = response.json()['data']
+        if 'due_date' not in response_data:
+            return
+        utc_date_string = response_data['due_date']
+        return datetime.datetime.strptime(utc_date_string, '%Y-%m-%dT%H:%M:%S').replace(tzinfo=datetime.timezone.utc)
+
+
 protocol = BackupProtocol
