@@ -3,11 +3,13 @@ from client.sources.common import core
 from client.sources.common import models
 from client.utils import format
 from client.utils import output
+from client.utils import storage
 import os
 
 ##########
 # Models #
 ##########
+from client.utils.printer import print_error
 
 
 class OkTest(models.Test):
@@ -179,13 +181,63 @@ class OkTest(models.Test):
         with open(test_tmp, 'w', encoding='utf-8') as f:
             f.write('test = {}\n'.format(json))
 
-        # Use an atomic rename operation to prevent test corruption
-        os.replace(test_tmp, self.file)
+        try:
+            storage.replace_transactional(test_tmp, self.file)
+        except (NotImplementedError, OSError):
+            # Try to use os.replace, but if on Windows manually remove then rename
+            # (ref issue #339)
+            if os.name == 'nt':
+            # TODO(colin) Add additional error handling in case process gets killed mid remove/rename
+                os.remove(self.file)
+                os.rename(test_tmp, self.file)
+            else:
+                # Use an atomic rename operation to prevent test corruption
+                os.replace(test_tmp, self.file)
 
     @property
     def unique_id_prefix(self):
         return self.assignment_name + '\n' + self.name
 
+    def get_code(self):
+        extracted_code = {}
+        for ind, suite in enumerate(list(self.suites)):
+            if suite.type != 'doctest':
+                continue
+            suite_code = suite.extract_code()
+            if suite_code:
+                # Store with 1 indexed name
+                extracted_code[ind+1] = suite_code
+        return extracted_code
+
+
+class EncryptedOKTest(models.Test):
+    name = core.String()
+    points = core.Float()
+    partner = core.String(optional=True)
+    def warn(self, method):
+        print_error("Cannot {} {}: test is encrypted".format(method, self.name))
+        keys_string = input("Please paste the key to decrypt this test: ")
+        keys = keys_string.strip().split()
+        if keys:
+            raise ex.ForceDecryptionException(keys)
+
+    def run(self, env):
+        self.warn('run')
+        return {'failed': 1, 'locked': 0, 'passed': 0}
+
+    def score(self):
+        self.warn('score')
+        return 0
+
+    def unlock(self, interact):
+        self.warn('unlock')
+
+    def lock(self, hash_fn):
+        self.warn('lock')
+
+    def dump(self):
+        self.warn('save the test')
+
 
 class Suite(core.Serializable):
     type = core.String()
@@ -225,6 +277,15 @@ class Suite(core.Serializable):
             return [x for x in enumerated if x[0] + 1 in self.run_only]
         return enumerated
 
+    def extract_code(self):
+        """Pull out the code for any doctest cases in the suite.
+        """
+        data = [{'setup': c.formatted_setup(),
+                 'code': c.formatted_code(),
+                 'teardown': c.formatted_teardown()} for _, c in self.enumerate_cases()
+                                                     if hasattr(c, 'setup')]
+        return data
+
     def _run_case(self, test_name, suite_number, case, case_number):
         """A wrapper for case.run().
 
@@ -251,9 +312,8 @@ class Suite(core.Serializable):
             print(''.join(output_log))
         if not success:
             short_name = self.test.get_short_name()
-            # TODO: Change when in notebook mode.
+            # TODO: Change when in notebook mode
             print('Run only this test case with '
                 '"python3 ok -q {} --suite {} --case {}"'.format(
                     short_name, suite_number, case_number))
-
         return success
